package com.xunlei.aplyer.aplayerdemo;

import android.Manifest;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.Toast;

import com.aplayer.APlayerAndroid;
import com.google.zxing.activity.CaptureActivity;
import com.xunlei.aplyer.aplayerdemo.model.Content;
import com.xunlei.aplyer.aplayerdemo.model.FileItemInfo;
import com.xunlei.aplyer.aplayerdemo.util.CommonConst;
import com.xunlei.aplyer.aplayerdemo.util.TimeUtil;
import com.xunlei.aplyer.aplayerdemo.view.FilePickerView;

public class FilePickerActivity extends AppCompatActivity {
	private static final String DEBUG_TAG = "FilePickerActivity";
	
	private static final String CUR_DIR_PATH = "CurDirPath";
	private String mCurDirPath;

	private FilePickerView mFilePickerView;
	private Context mContext;
	private Handler mHandler;
	private ProgressDialog mProgressDialog;

	public FilePickerActivity() {
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		Log.v(DEBUG_TAG, "onCreate");
		super.onCreate(savedInstanceState);
		mCurDirPath = getDefaultDir(savedInstanceState);
		mFilePickerView = new FilePickerView(this, mCurDirPath);
		mFilePickerView.setOnSelectListener(new FilePickerView.FilePickListener(){
			@Override
			public void onSelected(String filePath, boolean isFile) {
				if(isFile) {
					startPlayer(filePath);
				} else {
					mCurDirPath = filePath;
				}
			}
		});
		mFilePickerView.setOnMoreOperateListener(new FilePickerView.FileMoreOperateListener() {
			@Override
			public void onMoreSelected(FileItemInfo fileInfo, int operator) {
				switch (operator){
					case CommonConst.OPERATOR_EXTRACT_AUDIO:
						extractAudioTest(fileInfo);
						break;
					default:
						break;

				}
			}
		});

		setContentView(mFilePickerView);
		mContext = getApplicationContext();
		mHandler = new Handler();
		requestPermission();
		Log.e("mCurDirPath = ",mCurDirPath);
		ImageView scanButton = mFilePickerView.getmScanQRCodeBtn();
		if(scanButton != null){
			scanButton.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					startQrCode();
				}
			});
		}
	}

	@Override
	public void onSaveInstanceState(Bundle savedInstanceState) {
	    savedInstanceState.putString(CUR_DIR_PATH, mCurDirPath);
	    super.onSaveInstanceState(savedInstanceState);
		Log.e(DEBUG_TAG, "Save State");
	}


	private void startPlayer(String mediaFilePath)
	{
		Intent intent = new Intent(this, PlayerActivity.class);
		intent.setData(Uri.parse(mediaFilePath));
		intent.putStringArrayListExtra(CommonConst.PLAY_LIST_INTENT_KEY,mFilePickerView.getCurDirPlayableList());
		startActivity(intent);
	}

	private String getDefaultDir(Bundle savedInstanceState)
	{
		String defaultPath;
		if (savedInstanceState != null) {
			defaultPath = savedInstanceState.getString(CUR_DIR_PATH, Content.PATH_ROOT);
		}
		else
		{
			defaultPath = Environment.getExternalStorageDirectory().getAbsolutePath();
		}

		return defaultPath;
	}

	private void extractAudioTest(FileItemInfo fileInfo){
		if(fileInfo == null || fileInfo.isDirectory || fileInfo.mediaInfo== null /*|| fileInfo.mediaInfo.hasAudio==false*/){
			return;
		}
		if(mProgressDialog != null){
			mProgressDialog.dismiss();
			mProgressDialog = null;
		}
		mProgressDialog = new ProgressDialog(FilePickerActivity.this);
		mProgressDialog.setIndeterminate(false);//循环滚动
		mProgressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
		mProgressDialog.setMessage(getString(R.string.extract_audio_indicator));
		mProgressDialog.setCancelable(false);//false不能取消显示，true可以取消显示
		final String inPath = fileInfo.path;
		int lastDot = inPath.lastIndexOf(".");
		lastDot = lastDot == -1 ? inPath.length() : lastDot;
		final String outPath = inPath.substring(0,lastDot) + "_"+TimeUtil.getCurrentDate("")+".m4a";
		new Thread(new Runnable() {
			@Override
			public void run() {
				mHandler.post(new Runnable() {
					@Override
					public void run() {
						mProgressDialog.show();
					}
				});
				APlayerAndroid ap = new APlayerAndroid();
				ap.extractAudio(inPath,outPath);//async operation
				ap.setOnExtractAudioCompleteListener(new APlayerAndroid.OnExtractAudioCompleteListener() {
					@Override
					public void onExtractAudioComplete() {
						mProgressDialog.dismiss();
						Toast.makeText(mContext,"音频抽取完成："+outPath,Toast.LENGTH_LONG).show();
						mFilePickerView.updateFileList(mCurDirPath);
					}
				});
//				ap.close();
//				ap.destroy();
				//抽取音频还没有进度回调，先这样
				/*mHandler.postDelayed(new Runnable() {
					@Override
					public void run() {
						mProgressDialog.dismiss();
						Toast.makeText(mContext,"音频抽取完成："+outPath,Toast.LENGTH_LONG).show();
						mFilePickerView.updateFileList(mCurDirPath);
					}
				},15000);*/
			}
		}).start();


	}




	public static final int EXTERNAL_STORAGE_REQ_CODE = 10 ;

	public void requestPermission(){
		//判断当前Activity是否已经获得了该权限
		if (ContextCompat.checkSelfPermission(this,
				Manifest.permission.WRITE_EXTERNAL_STORAGE)
				!= PackageManager.PERMISSION_GRANTED) {

			//如果App的权限申请曾经被用户拒绝过，就需要在这里跟用户做出解释
			if (ActivityCompat.shouldShowRequestPermissionRationale(this,
					Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
				Toast.makeText(this,"please give me the permission",Toast.LENGTH_SHORT).show();
			} else {
				//进行权限请求
				ActivityCompat.requestPermissions(this,
						new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
						EXTERNAL_STORAGE_REQ_CODE);
			}
		}
	}

	@Override
	public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
		switch (requestCode) {
			case EXTERNAL_STORAGE_REQ_CODE:
				if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
					// Permission Granted
					mFilePickerView.updateFileList(mCurDirPath);
				} else {
					// Permission Denied
					Toast.makeText(FilePickerActivity.this, "WRITE_CONTACTS Denied", Toast.LENGTH_SHORT)
							.show();
				}
				break;
			case CommonConst.REQ_PERM_CAMERA:
				// 摄像头权限申请
				if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
					// 获得授权
					startQrCode();
				} else {
					// 被禁止授权
					Toast.makeText(FilePickerActivity.this, "请至权限中心打开本应用的相机访问权限", Toast.LENGTH_LONG).show();
				}
				break;
			default:
				super.onRequestPermissionsResult(requestCode, permissions, grantResults);
		}
	}

	private boolean mExit ;
	@Override
	public void onBackPressed() {

		if(!mExit){
			mHandler.postDelayed(new Runnable() {
				@Override
				public void run() {
					mExit = false;
				}
			},300);
			mExit = true;
			mFilePickerView.backToPreviousDir();
		} else {
			super.onBackPressed();
			finish();
		}


	}

	// 开始扫码
	private void startQrCode()
	{
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {
            // 申请权限
            ActivityCompat.requestPermissions(FilePickerActivity.this, new String[]{Manifest.permission.CAMERA}, CommonConst.REQ_PERM_CAMERA);
            return;
        }
        // 二维码扫码
        Intent intent = new Intent(FilePickerActivity.this, CaptureActivity.class);
        startActivityForResult(intent, CommonConst.REQ_QR_CODE);
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		//扫描结果回调
		if (requestCode == CommonConst.REQ_QR_CODE && resultCode == RESULT_OK) {
			Bundle bundle = data.getExtras();
			String scanResult = bundle.getString(CommonConst.INTENT_EXTRA_KEY_QR_SCAN);
			//将扫描出的信息显示出来
			//tvResult.setText(scanResult);
			startPlayer(scanResult);
		}
	}


}
