package com.xunlei.aplyer.aplayerdemo.view;

import android.content.Context;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.TextView;

import com.xunlei.aplyer.aplayerdemo.R;
import com.xunlei.aplyer.aplayerdemo.model.Content;
import com.xunlei.aplyer.aplayerdemo.play_interface.IConfigVideo;


/**
 * Created by LZ on 2016/9/24.
 */
public class AdvancedSetView extends LinearLayout {

    private static final String ERROR_TAG = Content.APLAYER_DEMO_LOG_PREF_TAG + AdvancedSetView.class.getSimpleName();

    private Context mContext;
    private View mRootView;
    private IConfigVideo mConfigVideo;

    TextView    mPlaySpeedCategory;
    TextView    mPlaySpeedVal;
    SeekBar mPlaySpeedSeekBar;

    //取值13,值为6时为1倍速起点，每增加/减少 一个单位，播放速度增加/减慢 一倍
    private static final int SEEK_BAR_MAX_RANGE = 13;
    private static final int NORMAL_SPEED_ORIGINE = 6;

    public AdvancedSetView(Context context, IConfigVideo configVideo) {
        super(context);

        mContext = context;
        mConfigVideo = configVideo;

        mRootView = LayoutInflater.from(context).inflate(R.layout.advanced_set_view, this);
        setOrientation(VERTICAL);

        init();
        updateParam();
        registerListener();
    }

    private void init()
    {
        mPlaySpeedCategory = (TextView) mRootView.findViewById(R.id.popwnd_advance_set);
        mPlaySpeedVal = (TextView) mRootView.findViewById(R.id.popwnd_advance_set_val);
        mPlaySpeedSeekBar = (SeekBar) mRootView.findViewById(R.id.popwnd_advance_set_seek_bar);

        mPlaySpeedSeekBar.setMax(SEEK_BAR_MAX_RANGE);
        mPlaySpeedSeekBar.setProgress(NORMAL_SPEED_ORIGINE);
    }


    private void registerListener() {
//        mRecordSwitch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
//            @Override
//            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
//                if(isChecked)
//                {
//                    String mediaPath = Environment.getExternalStorageDirectory() + "/" + getOutputFile(mOriginalMediaPath);
//                    if(! mRecord.startRecord(mediaPath))
//                    {
//                        Log.e(ERROR_TAG, "Start Record Failed!");
//                    }
//                }
//                else
//                {
//                    mRecord.endRecord();
//                }
//            }
//        });

        mPlaySpeedSeekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if(!fromUser) {
                    Log.e(ERROR_TAG, "Not for fromUser fromUser = " + fromUser);
                    return;
                }

                float playSpeed = seekBarePosToPlaySpeed(progress);
                setPlaySpeedText(playSpeed);
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

                float playSpeed = seekBarePosToPlaySpeed(seekBar.getProgress());
                setPlaySpeedText(playSpeed);
                if(!mConfigVideo.setPlaySpeed(playSpeed))
                {
                    Log.e(ERROR_TAG, "setPlaySpeed failed");
                }
            }
        });

    }

    private void updateParam()
    {
        float playSpeed = mConfigVideo.getPlaySpeed();
        setPlaySpeedText(playSpeed);

        int playSpeedSeekBarePos = playSpeedToseekBarePos(playSpeed);
        mPlaySpeedSeekBar.setProgress(playSpeedSeekBarePos);
    }

    private void setPlaySpeedText(float playSpeed)
    {
        String strPlaySpeed = floatToPercentageStr(playSpeed);
        mPlaySpeedVal.setText(strPlaySpeed);
    }

    private static String floatToPercentageStr(float playSpeed)
    {

        String strPlaySpeed = 100 * playSpeed + " %";
        return strPlaySpeed;
    }

    private static float seekBarePosToPlaySpeed(int pos)
    {
        int exponent = pos - NORMAL_SPEED_ORIGINE;
        final int BASE = 2;
        return  (float) Math.pow(BASE, exponent);
    }

    private int playSpeedToseekBarePos(float fPlaySpeed)
    {
        final int BASE = 2;
        double dOffset = Math.log(fPlaySpeed) / Math.log(BASE);
        int iOffset = (int)Math.round(dOffset);
        return NORMAL_SPEED_ORIGINE + iOffset;
    }
}